<?php
/**
 * Created by PhpStorm.
 * User: TREULIAUX
 * Date: 12/12/2016
 * Time: 18:10
 */

/**
 * @file
 * Contains \Drupal\media_upload\Form\MediaUploadConfigurationForm.
 */

namespace Drupal\media_upload\Form;

use Drupal\Component\Utility\Bytes;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Url;
use Drupal\field\Entity\FieldConfig;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;

/**
 * Contribute form.
 */
class MediaUploadConfigurationForm extends ConfigFormBase {

  const COMPLETE_FILE_NAME = 0;

  const FILE_NAME = 1;

  const TAG_NAME = 2;

  const EXT_NAME = 3;

  private $_mediaBundle = [
    'audio'    => FALSE,
    'video'    => FALSE,
    'document' => FALSE,
    'image'    => FALSE,
  ];

  private $_videoBundles = [];

  private $_audioBundles = [];

  private $_documentBundles = [];

  private $_imageBundles = [];

  public function __construct(
    \Drupal\Core\Config\ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($config_factory);

    $videoBundles = \Drupal::entityManager()
                           ->getStorage('media_bundle')
                           ->loadByProperties(['type' => 'video']);
    foreach ($videoBundles as $item) {
      $this->_videoBundles[$item->id] = "{$item->label} ({$item->id})";
    }

    $audioBundles = \Drupal::entityManager()
                           ->getStorage('media_bundle')
                           ->loadByProperties(['type' => 'audio']);
    foreach ($audioBundles as $item) {
      $this->_audioBundles[$item->id] = "{$item->label} ({$item->id})";
    }

    $documentBundles = \Drupal::entityManager()
                              ->getStorage('media_bundle')
                              ->loadByProperties(['type' => 'document']);
    foreach ($documentBundles as $item) {
      $this->_documentBundles[$item->id] = "{$item->label} ({$item->id})";
    }

    $imageBundles = \Drupal::entityManager()
                           ->getStorage('media_bundle')
                           ->loadByProperties(['type' => 'image']);
    foreach ($imageBundles as $item) {
      $this->_imageBundles[$item->id] = "{$item->label} ({$item->id})";
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {

    return 'mediaupload_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
    Request $request = NULL
  ) {

    $config = $this->config('media_upload.settings');
    $form['bundles'] = [
      '#type'          => 'checkboxes',
      '#title'         => $this->t('Accepted medias'),
      '#options'       => [
        'video'    => $this->t('Videos'),
        'image'    => $this->t('Images'),
        'document' => $this->t('Documents'),
        'audio'    => $this->t('Audios'),
      ],
      '#default_value' => $config->get('bundles'),
    ];
    $form['total_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max total size'),
      '#description' => t('Enter a value like "512" (bytes), "80 KB" (kilobytes) or "50 MB" (megabytes) in order to restrict the allowed file size. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('total_size'),
    ];
    $form['single_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max single file size'),
      '#description' => t('Enter a value like "512" (bytes), "80 KB" (kilobytes) or "50 MB" (megabytes) in order to restrict the allowed file size. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('single_size'),
    ];

    $form['videos'] = [
      '#type'   => 'fieldset',
      '#title'  => $this->t('Videos parameters'),
      '#states' => [
        'invisible' => [
          ':input[name="bundles[video]"]' => ['checked' => FALSE],
        ],
        'disabled'  => [
          ':input[name="bundles[video]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['videos']['video_bun'] = [
      '#type'          => 'select',
      '#validated'     => TRUE,
      '#title'         => $this->t('Video media bundle'),
      '#options'       => $this->_videoBundles,
      '#default_value' => $config->get('video_bundle'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[video]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[video]"]' => ['checked' => FALSE],
        ],
      ],
      '#ajax'          => [
        'callback' => 'Drupal\media_upload\Form\MediaUploadConfigurationForm::populateFields',
        'event'    => 'change',
      ],
    ];
    $form['videos']['video_fie'] = [
      '#type'          => 'select',
      '#validated'     => TRUE,
      '#title'         => $this->t('Video field'),
      '#options'       => $this->initFields($config->get('video_bundle')),
      '#default_value' => $config->get('video_field'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[video]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[video]"]' => ['checked' => FALSE],
        ],
      ],
      '#ajax'          => [
        'callback' => 'Drupal\media_upload\Form\MediaUploadConfigurationForm::populateValues',
        'event'    => 'change',
      ],
    ];
    $form['videos']['video_ext'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed videos extensions'),
      '#default_value' => $config->get('video_ext'),
      '#attributes'    => ['readonly' => 'readonly'],
    ];
    $form['videos']['video_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max video size'),
      '#default_value' => $config->get('video_size'),
      '#attributes'    => ['readonly' => 'readonly'],
    ];

    $form['images'] = [
      '#type'   => 'fieldset',
      '#title'  => $this->t('Images parameters'),
      '#states' => [
        'invisible' => [
          ':input[name="bundles[image]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['images']['image_bun'] = [
      '#type'          => 'select',
      '#validated'     => TRUE,
      '#title'         => $this->t('Image media bundle'),
      '#options'       => $this->_imageBundles,
      '#default_value' => $config->get('image_bundle'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[image]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[image]"]' => ['checked' => FALSE],
        ],
      ],
      '#ajax'          => [
        'callback' => 'Drupal\media_upload\Form\MediaUploadConfigurationForm::populateFields',
        'event'    => 'change',
      ],
    ];
    $form['images']['image_fie'] = [
      '#type'          => 'select',
      '#validated'     => TRUE,
      '#title'         => $this->t('Image field'),
      '#options'       => $this->initFields($config->get('image_bundle')),
      '#default_value' => $config->get('image_field'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[image]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[image]"]' => ['checked' => FALSE],
        ],
      ],
      '#ajax'          => [
        'callback' => 'Drupal\media_upload\Form\MediaUploadConfigurationForm::populateValues',
        'event'    => 'change',
      ],
    ];
    $form['images']['image_ext'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed images extensions'),
      '#default_value' => $config->get('image_ext'),
      '#attributes'    => ['readonly' => 'readonly'],
    ];
    $form['images']['image_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max image size'),
      '#default_value' => $config->get('image_size'),
      '#attributes'    => ['readonly' => 'readonly'],
    ];

    $form['documents'] = [
      '#type'   => 'fieldset',
      '#title'  => $this->t('Documents parameters'),
      '#states' => [
        'invisible' => [
          ':input[name="bundles[document]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['documents']['document_bun'] = [
      '#type'          => 'select',
      '#validated'     => TRUE,
      '#title'         => $this->t('Document media bundle'),
      '#options'       => $this->_documentBundles,
      '#default_value' => $config->get('document_bundle'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[document]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[document]"]' => ['checked' => FALSE],
        ],
      ],
      '#ajax'          => [
        'callback' => 'Drupal\media_upload\Form\MediaUploadConfigurationForm::populateFields',
        'event'    => 'change',
      ],
    ];
    $form['documents']['document_fie'] = [
      '#type'          => 'select',
      '#validated'     => TRUE,
      '#title'         => $this->t('Document field'),
      '#options'       => $this->initFields($config->get('document_bundle')),
      '#default_value' => $config->get('document_field'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[document]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[document]"]' => ['checked' => FALSE],
        ],
      ],
      '#ajax'          => [
        'callback' => 'Drupal\media_upload\Form\MediaUploadConfigurationForm::populateValues',
        'event'    => 'change',
      ],
    ];
    $form['documents']['document_ext'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed documents extensions'),
      '#default_value' => $config->get('document_ext'),
      '#attributes'    => ['readonly' => 'readonly'],
    ];
    $form['documents']['document_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max document size'),
      '#default_value' => $config->get('document_size'),
      '#attributes'    => ['readonly' => 'readonly'],
    ];

    $form['audios'] = [
      '#type'   => 'fieldset',
      '#title'  => $this->t('Audios parameters'),
      '#states' => [
        'invisible' => [
          ':input[name="bundles[audio]"]' => ['checked' => FALSE],
        ],
      ],
    ];
    $form['audios']['audio_bun'] = [
      '#type'          => 'select',
      '#validated'     => TRUE,
      '#title'         => $this->t('Audio media bundle'),
      '#options'       => $this->_audioBundles,
      '#default_value' => $config->get('audio_bundle'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[audio]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[audio]"]' => ['checked' => FALSE],
        ],
      ],
      '#ajax'          => [
        'callback' => 'Drupal\media_upload\Form\MediaUploadConfigurationForm::populateFields',
        'event'    => 'change',
      ],
    ];
    $form['audios']['audio_fie'] = [
      '#type'          => 'select',
      '#validated'     => TRUE,
      '#title'         => $this->t('Audio field'),
      '#options'       => $this->initFields($config->get('audio_bundle')),
      '#default_value' => $config->get('audio_field'),
      '#states'        => [
        'required' => [
          ':input[name="bundles[audio]"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="bundles[audio]"]' => ['checked' => FALSE],
        ],
      ],
      '#ajax'          => [
        'callback' => 'Drupal\media_upload\Form\MediaUploadConfigurationForm::populateValues',
        'event'    => 'change',
      ],
    ];
    $form['audios']['audio_ext'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed audios extensions'),
      '#default_value' => $config->get('audio_ext'),
      '#attributes'    => ['readonly' => 'readonly'],
    ];
    $form['audios']['audio_size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Max audio size'),
      '#default_value' => $config->get('audio_size'),
      '#attributes'    => ['readonly' => 'readonly'],
    ];

    $this->checkMediaBundles($form);

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $this->isInt('total_size', $form_state);
    $this->isInt('single_size', $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $config = $this->config('media_upload.settings');

    $config->set('bundles', $values['bundles']);
    $config->set('total_size', $values['total_size']);
    $config->set('single_size', $values['single_size']);

    if (!empty($values['bundles']['video'])) {
      $config->set('video_bundle', $values['video_bun']);
      $config->set('video_field', $values['video_fie']);
      $config->set('video_ext', preg_replace('/,|,?\s+/', ' ', $values['video_ext']));
      $config->set('video_size', $values['video_size']);
    }

    if (!empty($values['bundles']['image'])) {
      $config->set('image_bundle', $values['image_bun']);
      $config->set('image_field', $values['image_fie']);
      $config->set('image_ext', preg_replace('/,|,?\s+/', ' ', $values['image_ext']));
      $config->set('image_size', $values['image_size']);
    }

    if (!empty($values['bundles']['document'])) {
      $config->set('document_bundle', $values['document_bun']);
      $config->set('document_field', $values['document_fie']);
      $config->set('document_ext', preg_replace('/,|,?\s+/', ' ', $values['document_ext']));
      $config->set('document_size', $values['document_size']);
    }

    if (!empty($values['bundles']['audio'])) {
      $config->set('audio_bundle', $values['audio_bun']);
      $config->set('audio_field', $values['audio_fie']);
      $config->set('audio_ext', preg_replace('/,|,?\s+/', ' ', $values['audio_ext']));
      $config->set('audio_size', $values['audio_size']);
    }

    $config->save();
    drupal_set_message('Configuration done.');
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {

    return ['media_upload.settings'];
  }

  private function isInt($fieldName, FormStateInterface $form_state) {
    $val = $form_state->getValue($fieldName);
    if (empty($val)) {
      $val = 0;
    } else {
      $val = Bytes::toInt($val);
    }
    if (!is_numeric($val)) {
      $form_state->setErrorByName($fieldName, $this->t('The size must be a positive interger value'));
    }
    else {
      if (intval($val) < 0) {
        $form_state->setErrorByName($fieldName, $this->t('The size must be a positive interger value'));
      }
    }
  }

  private function checkMediaBundles(&$form) {
    foreach ($this->_mediaBundle as $item => $value) {
      if (empty(\Drupal::entityManager()
                       ->getStorage('media_bundle')
                       ->loadByProperties(['type' => $item]))
      ) {
        drupal_set_message($this->t("You don't have media bundle of the $item type. You should <a href=':link'>create one</a>", [
          ':link' => Url::fromRoute('entity.media_bundle.add_form')
                        ->toString(),
        ]), 'warning');
        $form['bundles'][$item]['#disabled'] = TRUE;
      }
    }
  }

  private function initFields($bundle = NULL) {
    if (!$bundle) {
      return [];
    }
    $fields = \Drupal::entityManager()->getFieldDefinitions('media', $bundle);
    $validFields = [];
    foreach ($fields as $field) {
      if (get_class($field) === 'Drupal\field\Entity\FieldConfig') {
        $validFields[$field->getName()] = $field->getLabel() . ' (' . $field->getName() . ')';
      }
    }

    return $validFields;
  }

  public function populateFields(&$form, $form_state) {
    $fields = \Drupal::entityManager()
                     ->getFieldDefinitions('media', $form_state->getTriggeringElement()['#value']);
    $validFields = [];
    foreach ($fields as $field) {
      if (get_class($field) === 'Drupal\field\Entity\FieldConfig' && "file" === $field->getType()) {
        $validFields[] = [
          'value' => $field->getName(),
          'text'  => $field->getLabel(),
        ];
      }
    }
    $selector = preg_replace('/bun.*/', 'fie', $form_state->getTriggeringElement()['#id']);
    $response = new AjaxResponse();
    $response->addCommand(new InvokeCommand("#$selector", 'empty'));
    $response->addCommand(new InvokeCommand("#$selector", 'html', [
      "<option value=\"\" selected=\"selected\">- Select -</option>",
    ]));
    foreach ($validFields as $item) {
      $response->addCommand(new InvokeCommand("#$selector", 'append', [
        "<option value='{$item['value']}'>{$item['text']} ({$item['value']})</option>",
      ]));
    }
    return $response;
  }

  public function populateValues(&$form, $form_state) {
    $bundleType = str_replace('_fie', '', $form_state->getTriggeringElement()['#name']);
    $field = FieldConfig::loadByName('media', $form[$bundleType.'s'][$bundleType.'_bun']['#value'], $form_state->getTriggeringElement()['#value']);

    $response = new AjaxResponse();
    $response->addCommand(new InvokeCommand("#edit-$bundleType-ext", 'val', [$field->getSetting('file_extensions')]));
    $response->addCommand(new InvokeCommand("#edit-$bundleType-size", 'val', [$field->getSetting('max_filesize')]));

    return $response;
  }
}
